from django.contrib.auth import get_user_model
from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse

from rest_framework import status
from rest_framework.test import APIClient

from django_drf_jwt.settings import api_settings


class JWTTests(TestCase):
    def setUp(self):
        self.username = "test"
        self.password = "test124356"
        self.user = User.objects.create(username=self.username)
        self.user.set_password(self.password)
        self.user.save()
        self.user_secret = getattr(self.user, api_settings.JWT_USER_SECRET_FIELD, None)
        self.client = APIClient()

        self.url = reverse("django_drf_jwt:get_token")
        self.revoke_url = reverse("django_drf_jwt:revoke_token")

    def test_jwt_auth_init(self):
        response = self.client.post(self.url, {})
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json().get("password")[0], "This field is required.")
        self.assertEqual(response.json().get("username")[0], "This field is required.")

    def test_jwt_auth_wrong_cred(self):
        response = self.client.post(self.url, {"username": "test", "password": "testing123"})
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json().get("non_field_errors")[0], "Unable to authenticate with provided credentials")

    def test_jwt_auth(self):
        response = self.client.post(self.url, {"username": self.username, "password": self.password})
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(len(response.json().get("token").split(".")), 3, "Expect JWT Token created")
        self.assertIn("created", response.json().keys(), "Expect created in response body")

    def test_revoke_token_init(self):
        response = self.client.post(self.revoke_url, {})
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_revoke_token(self):
        response = self.client.post(self.url, {"email": self.username, "password": self.password})
        token = response.json().get("token")
        self.client.credentials(HTTP_AUTHORIZATION="JWT " + token)

        response = self.client.post(self.revoke_url, {})
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        user = get_user_model().objects.get(email=self.username)
        self.assertNotEquals(self.user_secret, getattr(user, api_settings.JWT_USER_SECRET_FIELD, None))

        self.client.credentials(HTTP_AUTHORIZATION="JWT " + token)
        response = self.client.post(self.revoke_url, {})
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)



import uuid

from rest_framework import status
from rest_framework.generics import GenericAPIView
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response

from django_drf_jwt.serializers import JwtAuthSerializer
from django_drf_jwt.settings import api_settings


class JWTAuthView(GenericAPIView):
    permission_classes = (AllowAny,)
    authentication_classes = ()
    serializer_class = JwtAuthSerializer

    def post(self, request, *args, **kwargs):
        """
        Need to return token in response
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        token = serializer.validated_data["token"]
        created = serializer.validated_data["created"]
        return Response({"token": token, "created": created}, status=status.HTTP_201_CREATED)


class JWTRevokeTokenView(GenericAPIView):
    permission_classes = (IsAuthenticated,)

    def post(self, request, *args, **kwargs):
        new_secret = str(uuid.uuid4())
        setattr(request.user, api_settings.JWT_USER_SECRET_FIELD, new_secret)
        request.user.save(update_fields=[api_settings.JWT_USER_SECRET_FIELD])
        return Response({}, status=status.HTTP_201_CREATED)



from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ImproperlyConfigured
from rest_framework.settings import APISettings

USER_SETTINGS = getattr(settings, "JWT_DRF", None)

DEFAULTS = {
    "JWT_SECRET": settings.SECRET_KEY,
    "JWT_USER_ID_FIELD": "pk",
    "JWT_USER_SECRET_FIELD": None,  # MUST BE DEFINED - This must be a
    "JWT_PAYLOAD_HANDLER": "django_drf_jwt.handlers.payload_handler",
    "JWT_AUTH_HEADER_PREFIX": "JWT",
}

IMPORT_STRINGS = {
    "JWT_PAYLOAD_HANDLER",
}


api_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)

if not api_settings.JWT_USER_SECRET_FIELD:
    raise ImproperlyConfigured("JWT_USER_SECRET_FIELD must be set in settings file")


if not getattr(get_user_model(), api_settings.JWT_USER_SECRET_FIELD, None):
    raise ImproperlyConfigured("User don't have secret field")



import jwt
from django.contrib.auth import get_user_model
from rest_framework.authentication import BasicAuthentication
from rest_framework.exceptions import AuthenticationFailed

from django_drf_jwt.settings import api_settings


class JWTAuthentication(BasicAuthentication):
    """
    JWT authentication - django-drf-jwt authentication for user
    this class is responsible to authenticate user
    """

    @staticmethod
    def get_token(request):
        token = request.META.get("HTTP_AUTHORIZATION")
        if not token:
            return None

        token = token.split(" ")
        if len(token) != 2:
            return None
        if token[0].strip().lower() != api_settings.JWT_AUTH_HEADER_PREFIX.lower():
            return None
        return token[1]

    @classmethod
    def decode_token_from_header(cls, token: str) -> tuple[str, str]:
        if not token:
            raise AuthenticationFailed("No token provided")
        decoded_token = jwt.decode(token, api_settings.JWT_SECRET, algorithms=["HS256"])
        return decoded_token["user_id"], decoded_token["user_secret"]

    def authenticate(self, request):
        token = self.get_token(request)
        user_id, secret = self.decode_token_from_header(token)
        query = {api_settings.JWT_USER_ID_FIELD: user_id}
        try:
            user = get_user_model().objects.get(**query)
        except get_user_model().DoesNotExist:
            raise AuthenticationFailed("Could not validate credentials")

        if not self.user_can_authenticate(user):
            raise AuthenticationFailed("Could not validate credentials")

        # user_secret must be the same!
        if secret != str(getattr(user, api_settings.JWT_USER_SECRET_FIELD, None)):
            raise AuthenticationFailed("Could not validate credentials")
        return user, token

    @classmethod
    def user_can_authenticate(cls, user: any) -> bool:
        """This can be overwritten by your needs"""
        return user.is_active or user.is_superuser or user.is_staff



from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured

from django_drf_jwt.settings import api_settings


def payload_handler(user: User) -> dict[str, any]:
    """
    Create payload for authentication
    :param user:
    :return:
    """
    user_id_field = api_settings.JWT_USER_ID_FIELD
    if not user_id_field:
        user_id_field = "pk"
    user_secret_field = api_settings.JWT_USER_SECRET_FIELD
    if not user_secret_field:
        raise ImproperlyConfigured("JWT_USER_SECRET_FIELD must be defined in settings")
    if not getattr(user, user_secret_field, None):
        raise ImproperlyConfigured("User dont have specified secret field")

    return {
        "user_id": str(getattr(user, user_id_field, None)),
        "user_secret": str(getattr(user, user_secret_field, None)),
    }



import jwt
from django.contrib.auth import get_user_model, authenticate
from django.utils import timezone
from rest_framework import serializers

from django_drf_jwt.handlers import payload_handler
from django_drf_jwt.settings import api_settings


class JwtAuthSerializer(serializers.Serializer):

    password = serializers.CharField(write_only=True, required=True, style={"input_type": "password"})
    token = serializers.CharField(read_only=True)

    def __init__(self, *args, **kwargs):
        super(JwtAuthSerializer, self).__init__(*args, **kwargs)
        self.fields[self.username_field] = serializers.CharField(write_only=True, required=True)

    @property
    def username_field(self):
        return get_user_model().USERNAME_FIELD

    def validate(self, attrs):
        username = attrs.get(self.username_field)
        if not username:
            raise serializers.ValidationError("username is required")
        password = attrs.get("password", None)
        if not password:
            raise serializers.ValidationError("password is required")

        credentials = {self.username_field: username, "password": password}

        user = authenticate(**credentials)
        if not user:
            raise serializers.ValidationError("Unable to authenticate with provided credentials")

        token = jwt.encode(self.get_payload_handler(user), api_settings.JWT_SECRET, algorithm="HS256")
        return {
            "token": token,
            "created": timezone.now(),
        }

    @classmethod
    def get_payload_handler(cls, *args, **kwargs) -> dict[str, any]:
        return (
            api_settings.JWT_PAYLOAD_HANDLER(*args, **kwargs)
            if api_settings.JWT_PAYLOAD_HANDLER
            else payload_handler(*args, **kwargs)
        )



from django.urls import path

from django_drf_jwt.views import JWTAuthView, JWTRevokeTokenView

app_name = "django_drf_jwt"

urlpatterns = [
    path("get_token/", JWTAuthView.as_view(), name="get_token"),
    path("revoke/", JWTRevokeTokenView.as_view(), name="revoke_token"),
]
